import com.revusky.niggle.servlet.*;
import javax.servlet.http.*;
import java.io.IOException;
import java.util.*;

public class GuestbookServletInteraction extends ServletInteraction {

    // A container that contains a list of guestbook entries.
    // This variable must be static, since a new instance
    // of GuestbookServlet interaction is instantiated
    // on each request.
    static Vector entries = new Vector();


    /**
     * The obligatory constructor. Your ServletInteraction subclass must have a
     * constructor with this signature. This is the case even though we don't
     * actually have to do anything in the constructor (besides invoking
     * the superclass's constructor, that is.)
     */
    public GuestbookServletInteraction(HttpServletRequest request, 
                                       HttpServletResponse response, 
                                       NiggleConfig config) 
    throws IOException {
        super(request, response, config);
    }

    /**
     * The "default" action handler. This will be invoked
     * if there is no "action" parameter defined in the request parameters.
     * So the "default" action will be to display the list of 
     * guestbook entries.
     */
    public void execDefault() throws IOException {
        execEntries();
    }

    /**
     * This will be invoked if the parameter "action=entries"
     * is in the CGI parameters.
     */
    public void execEntries() throws IOException {
        this.page = getPage("entries.nhtml");
        page.expose("entries", entries); // expose the list of entries to the page.
    }

    /**
     * This will be invoked if the parameter "action=process"
     * is in the CGI parameters. This method processes
     * the form and adds the data to our data structure.
     */
    public void execProcess() throws IOException {
        this.page = getPage("thanks.nhtml");
        String name = getParameter("name");
        String email = getParameter("email");
        String comments = getParameter("comments");
        String remoteHost = request.getRemoteHost();
        if (!hasContent(name)) {
            page.expose("error", "The name field is required.");
        }
        else {
            Hashtable entry = new Hashtable();
            entry.put("name", name);
            entry.put("email", email);
            entry.put("comments", comments);
            entry.put("host", remoteHost);
            page.expose("entry", entry); // expose the entry to our page.
            this.entries.add(entry); // add the entry to our global list.
        }
    }

    /**
     * This will be invoked if the parameter "action=add"
     * is in the CGI parameters.
     */
    public void execAdd() throws IOException {
        this.page = getPage("add.nhtml");
    }

    /**
     * Add some extra default variables to be exposed
     * independently of what the action is.
     */
    protected void exposeDefaultVariables() throws IOException {
        super.exposeDefaultVariables();
        page.expose("entriesURL", getURL("entries"));
        page.expose("processURL", getURL("process"));
        page.expose("addURL", getURL("add"));
    }
}
