import com.revusky.niggle.servlet.*;
import javax.servlet.http.*;
import java.io.*;
import java.util.Locale;

public class I18NServletInteraction extends ServletInteraction {

    /**
     * The obligatory constructor. Your ServletInteraction subclass must have a
     * constructor with this signature. This is the case even though we don't
     * actually have to do anything in the constructor (besides invoking
     * the superclass's constructor, that is.)
     */
    public I18NServletInteraction(HttpServletRequest request, 
                                         HttpServletResponse response, 
                                         NiggleConfig config)
    throws IOException {
        super(request, response, config);
    }

    /**
     * Overrides the deduceLocale() hook
     * so that the locale is always fr_FR
     * This is for demonstration purposes.
     * uncomment the first line and comment out 
     * the second one for more sensible behavior.
     */
    protected void deduceLocale() {
        // Note that the request.getLocale() routine was added in the 2.2 servlet spec
        // locale = request.getLocale(); 
        locale = Locale.FRANCE;
//        locale = new Locale("es", "ES"); // Spain
    }


    /**
     * The "default" action handler. 
     * The page template is taken to be the XXXX.nhtml
     * that mapped onto this servlet.
     * The other difference between this example 
     * and helloniggle is that, rather than exposing literal strings,
     * we expose a ResourceBundle object.
     */
    public void execDefault() throws IOException {
        String servletPath = request.getServletPath();
        try {
            this.page = getPage(servletPath);
        }
        catch (FileNotFoundException fnfe) {
            this.page = getPage("error.nhtml");
        }
        page.exposeResourceBundle("strings"); //see strings*.properties
    }
}
