/*
Copyright (c) 2001, Jonathan Revusky, All rights reserved.
The original version of this source code is copyrighted and owned 
by Reminders On-Line, Inc. It was released under the terms 
of the BSD-style open source license below. The terms of that 
license also apply to the modified version here.
*/

/*
Copyright (c) 2000, Reminders On-Line, Inc. (ROL)
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

Neither the name of the Reminders On-Line nor the names of its contributors
may be used to endorse or promote products derived from this software
without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/

package com.revusky.niggle.data;

import com.revusky.niggle.util.*;
import java.io.*;
import java.util.*;

/**
 *  A convenient base class for DataSource implementations.  Implements some
 *  methods that are likely to be used by most.
 *
 *  @author <a href="mailto:jon@revusky.com">Jonathan Revusky</a>
 */

abstract public class AbstractDataSource 
// Uncomment the next line if you want to use Niggle's RMI capabilities.
// extends java.rmi.server.UnicastRemoteObject
implements DataSource {
    /**
     *  The name of this data source.
     */
    private String name;

    /**
     *  Flag indicating whether or not this data source has been initialized.
     */
    private boolean inited;

    /**
     * Our constructor must throw the exception
     * since the superclass's constructor does.
     */
    public AbstractDataSource() throws IOException {
    }

    public String getName() throws IOException {
        return name;
    }

    /**
     * TODO
     */
    public void init(Properties props) throws IOException {
        Assert.assert(!inited, "The init() method can only be called once.");
        this.name = props.getProperty("NAME");
        inited = true;
    }

    public final Record get(Object key) throws IOException {
        return get(null, key);
    }

    public final List keys() throws IOException {
        return keys(null);
    }
}
