/*
Copyright (c) 2001, Jonathan Revusky, All rights reserved.
The original version of this source code is copyrighted and owned 
by Reminders On-Line, Inc. It was released under the terms 
of the open source BSD-style license below. The terms of that 
license also apply to the modified version here.
*/

/*
Copyright (c) 2000, Reminders On-Line, Inc. (ROL)
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

Neither the name of the Reminders On-Line nor the names of its contributors
may be used to endorse or promote products derived from this software
without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/

package com.revusky.niggle.data;

import java.io.IOException;
import java.util.StringTokenizer;
import java.util.Locale;

/**
 * The base exception for niggle data.
 * This object optionally encapsulates a Record instance
 * which is the data record relevant to the exception thrown.
 * @author <a href="mailto:jon@revusky.com">Jonathan Revusky</a>
 */

public class DataException extends IOException {

    private Record record;

    private Exception nestedException;

    private Locale locale;


    public final Record getRecord() {
        return record;
    }

    public void setRecord(Record record) {
        this.record = record;
    }

    public Locale getLocale() {
        if (locale == null) {
            locale = Locale.getDefault();
        }
        return locale;
    }

    public void setLocale(Locale locale) {
        this.locale = locale;
    }

    public DataException(Record record, String message) {
        super(message);
        this.record = record;
    }

    public DataException(Record record, Exception nestedException) {
        super(nestedException.getMessage());
        this.record = record;
        this.nestedException = nestedException;
    }

    /**
     * Currently, this method is rather half-baked.
     * It only replaces the field name in a message
     * by its localized equivalent.
     * TODO: Some disposition for getting the localized
     * message pattern, i.e. "Le champ %1 est obligatoire."
     * for the French pattern of saying that a given field
     * is required.
     */

    public String getLocalizedMessage() {
        String message = getMessage();
        if (record == null)
            return message;
        RecordDescriptor fields = record.getMetadata();
        StringBuffer result = new StringBuffer();
        StringTokenizer st = new StringTokenizer(message, "\n\t\r:;, ", true);
        while (st.hasMoreTokens()) {
            String tok = st.nextToken();
            FieldDescriptor field = fields.getField(tok);
            if (field != null)
                tok = field.getLocalizedName(getLocale());
            result.append(tok);
        }
        return result.toString();
    }

    public Exception getNestedException() {
        return nestedException;
    }
}
