/*
Copyright (c) 2001, Jonathan Revusky, All rights reserved.
The original version of this source code is copyrighted and owned 
by Reminders On-Line, Inc. It was released under the terms 
of the BSD-style open source license below. The terms of that 
license also apply to the modified version here.
*/

/*
Copyright (c) 2000, Reminders On-Line, Inc. (ROL)
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

Neither the name of the Reminders On-Line nor the names of its contributors
may be used to endorse or promote products derived from this software
without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/

package com.revusky.niggle.data;

import java.io.IOException;
import java.util.List;
import com.revusky.niggle.data.metadata.DataSourceMetadata;

/**
 * The interface implemented by objects that vend "virgin" instances
 * of a Record. A virgin instance is "vended" based on the type and version
 * number passed in as arguments to the getExemplar() method.
 * This is a creational pattern called the exemplar or
 * canonical object pattern IIRC.
 * @author <a href="mailto:jon@revusky.com">Jonathan Revusky</a>
 */
public interface DataRegistry extends java.rmi.Remote {

    /**
     * Register the metadata for a record type.
     */
    void registerRecordType(RecordDescriptor descriptor) throws IOException;

    /**
     * @return a "virgin" record of the given type and version.
     * If the right version is not found, then it returns the
     * exemplar of the highest available version number.
     * If nothing is available, it returns null.
     */
    Record getExemplar(String type, int version) throws IOException;

    /**
     * @return a "virgin" record of the given type and the highest version
     * available.
     */
    Record getExemplar(String type) throws IOException;

    /**
     * @return a DataSource by its unique name
     */
    DataSource getDataSource(String name) throws IOException;

    /**
     * register a DataSource by its unique name
     */
    void registerDataSource(DataSource dataSource) throws IOException;

    /**
     * register the metadata for a DataSource by its
     * unique name.
     */
    void registerDataSource(DataSourceMetadata dsm) throws IOException;

    /**
     * get record Metadata for the type and version.
     */
    RecordDescriptor getMetadata(String type, int version) throws IOException;

    /**
     * get record Metadata for the type and the highest version number available.
     */
    RecordDescriptor getMetadata(String type) throws IOException;

    /**
     * @return a List of the names of all of the 
     * record types that this data registry knows about.
     */

    List getRecordNames() throws IOException;

    /**
     * @return a List of the names of all of the 
     * data sources that this data registry knows about.
     */

    List getDataSourceNames() throws IOException;
}
