/*
Copyright (c) 2001, Jonathan Revusky, All rights reserved.
The original version of this source code is copyrighted and owned 
by Reminders On-Line, Inc. It was released under the terms 
of the BSD-style open source license below. The terms of that 
license also apply to the modified version here.
*/

/*
Copyright (c) 2000, Reminders On-Line, Inc. (ROL)
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

Neither the name of the Reminders On-Line nor the names of its contributors
may be used to endorse or promote products derived from this software
without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/

package com.revusky.niggle.data;

import java.io.*;

/**
 * An implementation of ObjectOutput for spitting out
 * niggle records onto a stream in their default flat-file format.
 * @see DefaultRecordInputStream for more info on the stream format.
 * @author <a href="mailto:jon@revusky.com">Jonathan Revusky</a>
 */
public class DefaultRecordOutputStream extends DataOutputStream implements ObjectOutput {

    /**
     * Serialise object in ASCII format xx=yyy
     * @param o Record to write
     */
    public void writeObject(Object o) throws IOException {
        if ( !(o instanceof Record) ) {
            throw new InvalidDataException(
                                          "Tried to store object of type "
                                          + o.getClass().getName()
                                          + "\nMust implement com.revusky.niggle.data.Record");
        }
        DefaultRecord rec = (DefaultRecord) o;
        // TODO: Some version checking.
        // Just ignore deleted or uninitialized records.
        if ( !rec.isStale() && rec.isImmutable() )
            rec.writeExternal(this);
    }


    /**
     * @param os the underlying outputstream to write to.
     */
    public DefaultRecordOutputStream(OutputStream os) {
        super(os);
    }

    /**
     * A convenience constructor that takes a filename
     * argument.
     * @param filename where to squirt the serialised stream.
     */
    public DefaultRecordOutputStream(String filename) throws IOException {
        this(new BufferedOutputStream(new FileOutputStream(filename), 2048));
    }
}
