/*
Copyright (c) 2001, Jonathan Revusky, All rights reserved.
The original version of this source code is copyrighted and owned 
by Reminders On-Line, Inc. It was released under the terms 
of the BSD-style open source license below. The terms of that 
license also apply to the modified version here.
*/

/*
Copyright (c) 2000, Reminders On-Line, Inc. (ROL)
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

Neither the name of the Reminders On-Line nor the names of its contributors
may be used to endorse or promote products derived from this software
without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/

package com.revusky.niggle.data.metadata;

import com.revusky.niggle.data.*;
import com.revusky.niggle.util.*;
import org.w3c.dom.*;
import org.xml.sax.SAXException;
import java.io.IOException;
import java.util.*;

/**
 * A String field that only admits a set
 * of possible string values.
 * @author <a href="mailto:jon@revusky.com">Jonathan Revusky</a>
 */

public class EnumField extends StringField {

    private String[] possibleValues;

    protected void init(Element elem) throws SAXException {
        NodeList elements = elem.getElementsByTagName("ELEMENT");
        ArrayList l = new ArrayList();
        for (int i = 0; i<elements.getLength(); i++) {
            Element e = (Element) elements.item(i);
            String text = DOMUtil.getText(e);
            if (l.contains(text))
                throw new SAXException(
                                      "All of the elements in an Enum field must be unique. "
                                      + text + " is repeated.");
            l.add(text.intern());
        }
        this.possibleValues = new String[l.size()];
        l.toArray(possibleValues);
        super.init(elem);
        if (!Util.hasContent(sqlType)) {
            StringBuffer buf = new StringBuffer();
            for (int i = 0; i< possibleValues.length; i++) {
                if (i==0)
                    buf.append("ENUM(");
                else
                    buf.append(", ");
                buf.append((char) '\"');
                buf.append(possibleValues[i]);
                buf.append((char) '\"');
            }
            buf.append((char) ')');
            sqlType = buf.toString();
        }
    }

    public Object getObjectFromString(Record rec, String value) throws MangledDataException {
        if (!Util.hasContent(value))
            return null;
        int index = indexOf(value);
        if (index <0)
            throw new MangledDataException(rec, "Field " + getName()
                                           + ", value "
                                           + value + " is not a valid value.");
        return possibleValues[index];
    }

    public void checkValidValue(Record rec, Object value) throws IOException {
        super.checkValidValue(rec, value);
        if (Util.hasContent((String) value) && (indexOf((String) value) <0))
            throw new MangledDataException(rec,
                                           "Field " + getName()
                                           + ", value "
                                           + value + " is not a valid value.");
    }

    private int indexOf(String value) {
        for (int i=0; i< possibleValues.length; i++) {
            if (possibleValues[i] == value)
                return i;
        }
        for (int i=0; i< possibleValues.length; i++) {
            if (possibleValues[i].equals(value))
                return i;
        }
        return -1;
    }

    public String[] getPossibleValues() {
        return(String[]) possibleValues.clone();
    }
}
