/*
Copyright (c) 2001, Jonathan Revusky, All rights reserved.
The original version of this source code is copyrighted and owned 
by Reminders On-Line, Inc. It was released under the terms 
of the BSD-style open source license below. The terms of that 
license also apply to the modified version here.
*/

/*
Copyright (c) 2000, Reminders On-Line, Inc. (ROL) 
All rights reserved.

Redistribution and use in source and binary forms, with or without 
modification, are permitted provided that the following conditions
are met:

Redistributions of source code must retain the above copyright notice, 
this list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice, 
this list of conditions and the following disclaimer in the documentation 
and/or other materials provided with the distribution.

Neither the name of the Reminders On-Line nor the names of its contributors 
may be used to endorse or promote products derived from this software 
without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE. 
*/

package com.revusky.niggle.data.metadata;

import com.revusky.niggle.data.*;
import com.revusky.niggle.util.*;
import java.util.*;

/**
 * A Field subtype that indicates that this field represents
 * an instance of java.util.Locale. At a later point,
 * we may add provision for limiting the valid locales to
 * a given subset of currently supported languages etc.
 * @author <a href="mailto:jon@revusky.com">Jonathan Revusky</a>
 */
public class LocaleField extends Field {

    static final Class localeClass = Locale.class;

    public Class getJavaType() {
        return localeClass;
    }

    public Object getObjectFromString(Record rec, String value) throws MangledDataException {
        if (!Util.hasContent(value))
            return null; // This should maybe get massaged into a Locale.getDefault() I guess.
        StringTokenizer st = new StringTokenizer(value, "_");
        String lang = st.nextToken();
        String country = "";
        String variant = "";
        if (st.hasMoreTokens()) {
            country = st.nextToken();
            if (st.hasMoreTokens())
                variant = st.nextToken();
        }
        // TODO: Check that the lang is a two-character ISO lang code etc.
        return new Locale(lang, country, variant);
    }

    public Object convertFromJDBC(Object o) {
        if (o == null)
            return null;
        String s = (String) o;
        StringTokenizer st = new StringTokenizer(s, "_");
        String[] ss = new String[3];
        for (int i=0; i<3;i++) {
            ss[i] = "";
        }
        for (int i = 0; i<3; i++) {
            if (st.hasMoreTokens()) {
                ss[i] = st.nextToken();
            }
            else 
                break;
        }
        return new Locale(ss[0], ss[1], ss[2]);
    }
}
