/*
Copyright (c) 2001, Jonathan Revusky, All rights reserved.
The original version of this source code is copyrighted and owned 
by Reminders On-Line, Inc. It was released under the terms 
of the BSD-style open source license below. The terms of that 
license also apply to the modified version here.
*/

/*
Copyright (c) 2000, Reminders On-Line, Inc. (ROL)
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

Neither the name of the Reminders On-Line nor the names of its contributors
may be used to endorse or promote products derived from this software
without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/

package com.revusky.niggle.data.metadata;

import org.w3c.dom.*;
import org.xml.sax.SAXException;
import org.xml.sax.InputSource;
import com.revusky.niggle.util.*;
import com.revusky.niggle.data.*;
import java.io.*;
import java.util.*;

abstract public class MetadataLoader {

    static public void loadRecordMetadata(DataRegistry registry, String filename)
    throws SAXException, IOException {
        loadRecordMetadata(registry, filename, null);
    }

    static public void loadDataSources(DataRegistry registry, String filename)
    throws SAXException, IOException {
        loadDataSources(registry, filename, null);
    }

    static public void loadRecordMetadata(DataRegistry registry, String filename, ClassLoader cl)
    throws SAXException, IOException {
        Document doc = getDOM(filename, cl);
        Map recordData = new HashMap();
        NodeList recNodes = doc.getElementsByTagName("RECORD");
        for (int i = 0; i<recNodes.getLength(); i++) {
            Element element = (Element) recNodes.item(i);
            String typeInfo = null;
            String unqualifiedType = null;
            int version = 0;
            unqualifiedType = element.getAttribute("TYPE");
            version = Integer.parseInt(element.getAttribute("VERSION"));
            Assert.hasContent(unqualifiedType);
            Assert.assert(version > 0);
            typeInfo = unqualifiedType + "." + version;
            if (recordData.containsKey(typeInfo))
                throw new SAXException("The combination of type and version must be unique. "
                                       + typeInfo + " has already occurred.");
            RecordMetadata data = new RecordMetadata();
            data.init(element, Collections.unmodifiableMap(recordData), cl);
            recordData.put(typeInfo, data);
            recordData.put(unqualifiedType, data);
        }
        for (Iterator it = recordData.values().iterator(); it.hasNext();) {
            registry.registerRecordType((RecordMetadata) it.next());
        }
    }

    static public void loadDataSources(DataRegistry registry, String filename, ClassLoader cl)
    throws SAXException, IOException {
        Document doc = getDOM(filename, cl);
        NodeList nodes = doc.getElementsByTagName("DATASOURCE");
        for (int i = 0; i<nodes.getLength(); i++) {
            Element element = (Element) nodes.item(i);
            DataSourceMetadata dsm = new DataSourceMetadata();
            dsm.init(element, cl);
            registry.registerDataSource(dsm);
        }
    }

    static public Document getDOM(String filename, ClassLoader cl)
    throws IOException, SAXException {
        try {
            return getProjectXDOM(filename, cl);
        }
        catch (Throwable t) {
            return getXercesDOM(filename, cl);
        }

    }

    static private Document getXercesDOM(String filename, ClassLoader cl)
    throws IOException, SAXException {
        InputStream is = Util.getInputStream(filename, cl);
        if (is == null)
            throw new FileNotFoundException("could not find " + filename);
        org.apache.xerces.parsers.DOMParser parser = new org.apache.xerces.parsers.DOMParser();
        parser.setEntityResolver(new ClasspathEntityResolver(cl));
        InputSource source = new InputSource(is);
        source.setSystemId("classpath:" + filename);
        parser.parse(source);
        return parser.getDocument();
    }
    /*
    static private Document getXercesDOM(String filename, ClassLoader cl)
    throws IOException, SAXException
    {
        throw new UnsupportedOperationException();
    }
    */

    static private Document getProjectXDOM(String filename, ClassLoader cl) {
        throw new UnsupportedOperationException();
    }

//For some reason, this method doesn't work. I don't know why.
/*
    static private Document getProjectXDOM(String filename, ClassLoader cl)
    throws IOException, SAXException
    {
        InputStream is = Util.getInputStream(filename, cl);
        if (is == null)
            throw new FileNotFoundException("could not find " + filename);
        javax.xml.parsers.DocumentBuilderFactory factory;
        javax.xml.parsers.DocumentBuilder builder = null;
        factory = javax.xml.parsers.DocumentBuilderFactory.newInstance();
        try {
            builder = factory.newDocumentBuilder();
        } catch (Exception e) {
            throw new AssertionFailedException(e);
        }
        builder.setEntityResolver(new ClasspathEntityResolver(cl));
        InputSource source = new InputSource(is);
        source.setSystemId("classpath:" + filename);
        Document result = builder.parse(source);
        return result;
    } 
*/    
}
