/*
Copyright (c) 2001, Jonathan Revusky
All rights reserved.

Redistribution and use in source and binary forms, with or without 
modification, are permitted provided that the following conditions
are met:

Redistributions of source code must retain the above copyright notice, 
this list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice, 
this list of conditions and the following disclaimer in the documentation 
and/or other materials provided with the distribution.

The name of Jonathan Revusky may not be used to endorse or promote 
products derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE. 
*/

package com.revusky.niggle.data.transaction;

import com.revusky.niggle.data.*;
import java.io.IOException;
import java.util.List;

/**
 * An interface that represents the contract of a Niggle DataSource
 * that supports atomic transactions
 */

public interface TransactionalDataSource extends MutableDataSource {

    /**
     * update the Record taking into account the transaction which this is part of.
     */
    void insert(Record rec, String transactionID) throws IOException;

    /**
     * update the Record taking into account the transaction which this is part of.
     */
    void update(Record rec, String transactionID) throws IOException;

    /**
     * delete the Record of the given type and primary key
     * taking into account the transaction which this is part of.
     */
    void delete(String type, Object key, String transactionID) throws IOException;

    /**
     * @return the Record of the given type and primary key
     * taking into account the transaction which this is part of.
     */
    Record get(String type, Object key, String transactionID) throws IOException;

    /**
     * @return a List of records that meet the criterion encapsulated by
     * the filter arg, taking into account the transaction which this is part of.
     */
    List select(String type, RecordFilter filter, String transactionID) throws IOException;

    /**
     * Commit the transaction that is identified by this transactionID
     */
    void commit(String transactionID) throws IOException;

    /**
     * Roll back the transaction 
     * @parm the unique ID that identifies this transaction
     */
    void rollback(String transactionID) throws IOException;

    /**
     *@return a unique ID for a new transaction.
     */
    String newTransaction() throws IOException;
}
