/*
Copyright (c) 2001, Jonathan Revusky, All rights reserved.
The original version of this source code is copyrighted and owned 
by Reminders On-Line, Inc. It was released under the terms 
of the BSD-style open source license below. The terms of that 
license also apply to the modified version here.
*/

/*
Copyright (c) 2000, Reminders On-Line, Inc. (ROL)
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

Neither the name of the Reminders On-Line nor the names of its contributors
may be used to endorse or promote products derived from this software
without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/

package com.revusky.niggle.templates;

import com.revusky.niggle.data.*;
import com.revusky.niggle.servlet.ServletInteraction;
import com.revusky.niggle.util.*;
import java.util.Locale;

abstract public class PresentationWrapper {

    static protected Class recordWrapperClass = com.revusky.niggle.templates.freemarkerimpl.RecordWrapper.class;

    static public void setRecordWrapperClass(Class cl) {
        Assert.assert(PresentationWrapper.class.isAssignableFrom(cl));
        recordWrapperClass = cl;
    }

    protected ServletInteraction interaction;
    protected String contentType = "text/html";

    /**
     * locale to use to select encoding for generated HTML
     */
    protected Locale locale = Locale.getDefault();


    public void setSessionContext(ServletInteraction interaction) {
        this.interaction = interaction;
        if (interaction != null)
            this.locale = interaction.getLocale();
    }

    public void setContentType(String contentType) {
        this.contentType = contentType;
    }

    protected void inheritFrom(PresentationWrapper pw) {
        this.contentType = pw.contentType;
        this.locale = pw.locale;
        this.interaction = pw.interaction;
    }

    /**
     * create a wrapper of the appropriate class
     * to wrap the given record, reusing the context info.
     */
    static protected PresentationWrapper createRecordWrapper(Record rec, ServletInteraction interaction, String contentType) {
        PresentationWrapper result =null;
        RecordDescriptor desc = rec.getMetadata();
        Class cl = desc.getPresentationWrapperClass();
        if (cl == null) // default to recordWrapperClass
            cl = recordWrapperClass;
        try {
            result = (PresentationWrapper) cl.newInstance();
        }
        catch ( Exception e ) {
            throw new AssertionFailedException("Problem instantiating class. "
                                               + cl.getName()
                                               + "\nNote that the class must inherit "
                                               + "from RecordWrapper and it  "
                                               + "must be public and have a public "
                                               +  "no-args constructor",
                                               e);
        }
        result.setRecord(rec);
        result.setSessionContext(interaction);
        result.setContentType(contentType);
        return result;
    }

    public void setRecord(Record rec) {
        throw new UnsupportedOperationException();
    }

    public Locale getLocale() {
        return locale;
    }
}
