/*
Copyright (c) 2001, Jonathan Revusky, All rights reserved.
The original version of this source code is copyrighted and owned 
by Reminders On-Line, Inc. It was released under the terms 
of the BSD-style open source license below. The terms of that 
license also apply to the modified version here.
*/

/*
Copyright (c) 2000, Reminders On-Line, Inc. (ROL)
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

Neither the name of the Reminders On-Line nor the names of its contributors
may be used to endorse or promote products derived from this software
without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/

package com.revusky.niggle.templates.freemarkerimpl;

import java.io.*;
import java.util.*;
import javax.servlet.ServletContext;
import freemarker.template.*;
import com.revusky.niggle.servlet.*;
import com.revusky.niggle.templates.*;
import com.revusky.niggle.util.*;

/**
 * A concrete PageFactory type that vends Page objects
 * that represent Freemarker templates.
 * @author <a href="mailto:jon@revusky.com">Jonathan Revusky</a>
 */

public class FreemarkerPageFactory implements PageFactory {

    protected LocalizedFileTemplateCache templateCache = new LocalizedFileTemplateCache();
    protected File defaultDirectory;
    private ClassLoader classLoader;

    public void setDefaultPath(String path) {
        File dir = new File(path);
        if (dir.isAbsolute()) {
            Assert.assert(dir.canRead(), "cannot read directory: " + dir);
            defaultDirectory = dir;
        }
        templateCache.setPath(path);
    }

    public Page getPage(String name, ServletInteraction interaction) throws IOException {
        LazyHash dataModel = new LazyHash();
        dataModel.setSessionContext(interaction);
        File file = new File(name);
        String contentType = contentTypeFromFilename(name);
        Template template = null;
        if ((templateCache != null) && (!file.isAbsolute())) {
            template = templateCache.getTemplate(name, interaction.getLocale());
        }
        else {
            if (!file.isAbsolute()) {
                if (defaultDirectory == null) {
                    throw new FileNotFoundException("The filename: "
                                                    + name
                                                    + " is relative but no defaultDirectory was specified.");
                }
                // Build a new file with the absolute path.
                file = new File(defaultDirectory, name);
            }
            template = new Template(file);
        }
        if (template == null) {
            throw new FileNotFoundException("Template: " + name + " not found");
        }
        return new FreemarkerPage(
                                  template, 
                                  dataModel,
                                  interaction.getClass().getClassLoader(),
                                  contentType);
    }


    public void setClassLoader(ClassLoader cl) {
        this.classLoader = cl;
        templateCache.setClassLoader(cl);
    }


    String contentTypeFromFilename(String filename) {
        filename = filename.toLowerCase();
        if (filename.endsWith("html") || filename.endsWith("htm"))
            return "text/html";
        if (filename.endsWith(".txt"))
            return "text/plain";
        if (filename.endsWith(".xml"))
            return "text/xml";
        return "text/html";
    }
}
