/*
Copyright (c) 2001, Jonathan Revusky, All rights reserved.
The original version of this source code is copyrighted and owned 
by Reminders On-Line, Inc. It was released under the terms 
of the BSD-style open source license below. The terms of that 
license also apply to the modified version here.
*/

/*
Copyright (c) 2000, Reminders On-Line, Inc. (ROL)
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

Neither the name of the Reminders On-Line nor the names of its contributors
may be used to endorse or promote products derived from this software
without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/

package com.revusky.niggle.templates.freemarkerimpl;

import java.util.*;
import java.text.DateFormat;
import com.revusky.niggle.data.*;
import com.revusky.niggle.servlet.ServletInteraction;
import com.revusky.niggle.templates.*;
import com.revusky.niggle.util.*;
import freemarker.template.TemplateModel;

abstract public class FreemarkerPresentationWrapper extends PresentationWrapper {

    // change this constant to false for case sensitivity.
    public static final boolean CASE_INSENSITIVE = true;

    protected TemplateModel createWrapper(Object val) {
        if (val instanceof TemplateModel) {
            return(TemplateModel) val;
        }
        if (val == null || Boolean.FALSE.equals(val)) {
            return null;
        }
        if (val instanceof Record) {
            return(TemplateModel) createRecordWrapper((Record) val, interaction, contentType);
        }
        if (val instanceof List) {
            return new LazyList((List) val, interaction, contentType);
        }
        if (val instanceof Map) {
            return createNestedHash((Map) val);
        }
        if (val instanceof Date) {
            val = DateFormat.getDateInstance(DateFormat.MEDIUM, locale).format((Date) val);
        }
        return wrapString(val.toString());
    }

    protected LazyHash createNestedHash(Map map) {
        LazyHash nested = new LazyHash();
        nested.inheritFrom(this);
        nested.hash = normalizeMap(map);
        nested.locale = this.locale;
        nested.interaction = interaction;
        return nested;
    }

    // make sure that the nested hashmap has only
    // lower case strings as keys.

    protected HashMap normalizeMap(Map m) {
        HashMap result = new HashMap();
        for (Iterator it = m.keySet().iterator(); it.hasNext();) {
            Object key = it.next();
            if (key instanceof String) {
                String lc = key.toString();
                if (CASE_INSENSITIVE)
                    lc = lc.toLowerCase();
                result.put(lc, m.get(key));
            }
        }
        return result;
    }


    protected TemplateModel wrapString(String s) {
        return new SmartScalar(s, contentType.equals("text/html"));
    }
}
