/*
Copyright (c) 2001, Jonathan Revusky
All rights reserved.

Redistribution and use in source and binary forms, with or without 
modification, are permitted provided that the following conditions
are met:

Redistributions of source code must retain the above copyright notice, 
this list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice, 
this list of conditions and the following disclaimer in the documentation 
and/or other materials provided with the distribution.

The name of Jonathan Revusky may not be used to endorse or promote 
products derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE. 
*/

package com.revusky.niggle.templates.freemarkerimpl;

import freemarker.template.*;
import java.util.*;


/**
 * A freemarker template model specifically designed
 * to "wrap" a Resource bundle containing externalized strings
 * @author <a href="mailto:jon@revusky.com">Jonathan Revusky</a>
 */

class ResourceBundleHash implements TemplateHashModel {

    private ResourceBundle bundle;

    ResourceBundleHash(ResourceBundle bundle) {
        this.bundle = bundle;
    }

    public TemplateModel get(String key) throws TemplateModelException {
        try {
            String s = null;
            try {
                s = bundle.getString(key);
            }
            catch (MissingResourceException mre) {
                if (key.indexOf('_') >=0)
                    s = bundle.getString(key.replace('_', '.'));
                else 
                    throw mre;
            }
            return new SmartScalar(s);
        }
        catch (RuntimeException re) {
            throw new TemplateModelException(re.getMessage());
        }
    }

    public boolean isEmpty() {
        return (bundle == null)
            || !bundle.getKeys().hasMoreElements();
    }
}
