/*
Copyright (c) 2001, Jonathan Revusky, All rights reserved.
The original version of this source code is copyrighted and owned 
by Reminders On-Line, Inc. It was released under the terms 
of the BSD-style open source license below. The terms of that 
license also apply to the modified version here.
*/

/*
Copyright (c) 2000, Reminders On-Line, Inc. (ROL)
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

Neither the name of the Reminders On-Line nor the names of its contributors
may be used to endorse or promote products derived from this software
without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/

package com.revusky.niggle.util;

/**
 * Runtime exception to indicate that an assertion
 * has failed. Now allows wrapping of exceptions,
 * useful for tunnelling of a checked exception in
 * an assertion.
 * @author <a href="mailto:jon@revusky.com">Jonathan Revusky</a>
 */
public class AssertionFailedException extends RuntimeException {

    public AssertionFailedException() {
        super("Assertion failed.");
    }

    public AssertionFailedException(String message) {
        super((message == null) ? "Assertion failed: " : message);
    }

    /**
     * Create a new AssertionFailedException wrapping an existing exception.
     *
     * <p>The existing exception will be embedded in the new
     * one, and its message will become the default message for
     * the AssertionFailedException.</p>
     *
     * @param t The exception or error to be wrapped in an AssertionFailedException.
     */
    public AssertionFailedException (Throwable t) {
        this(t.getMessage());
        this.nested = t;
    }


    /**
     * Create a new AssertionFailedException from an existing exception.
     *
     * <p>The existing exception will be embedded in the new
     * one, but the new exception will have its own message.</p>
     *
     * @param message The detail message.
     * @param t The exception/error to be wrapped in an AssertionFailedException.
     */
    public AssertionFailedException (String message, Throwable t) {
        this(message);
        this.nested = t;
    }

    /**
     * the wrapped exception (or null)
     */

    public Throwable getNested() {
        return nested;
    }

    private Throwable nested;
}
